@@ -336,6 +336,9 @@ public final class IMAPConversationWorker {
                 fp = Conversations.getFetchProfileConversationByHeaders(null == sortField ? MailField.RECEIVED_DATE : MailField.toField(sortField.getListField()));
             }
             conversations = Conversations.conversationsFor(imapMessageStorage.getImapFolder(), lookAhead, order, fp, imapMessageStorage.getImapServerInfo(), byEnvelope);
+            if (conversations.isEmpty()) {
+                return Collections.emptyList();
+            }
             // Retrieve from sent folder
             int sentTotal = 0;
             long sentUidNext = 0L;
@@ -353,29 +356,19 @@ public final class IMAPConversationWorker {
                         for (final Conversation conversation : conversations) {
                             conversation.addMessageIdsTo(allMessageIds);
                         }
-                        List<MailMessage> tmp = null;
-                        for (MailMessage sentMessage : sentMessages) {
-                            if (false == allMessageIds.contains(sentMessage.getMessageId())) {
-                                if (null == tmp) {
-                                    tmp = new LinkedList<>();
-                                }
-                                tmp.add(sentMessage);
+                        for (Iterator<MailMessage> iter = sentMessages.iterator(); iter.hasNext();) {
+                            MailMessage sentMessage = iter.next();
+                            if (allMessageIds.contains(sentMessage.getMessageId())) {
+                                // Already contained
+                                iter.remove();
                             }
                         }
-                        if (null != tmp) {
-                            sentMessages = tmp;
-                        }
                     }
 
                     if (false == sentMessages.isEmpty()) {
                         // Add to conversation if references or referenced-by
-                        for (Conversation conversation : conversations) {
-                            for (MailMessage sentMessage : sentMessages) {
-                                if (conversation.referencesOrIsReferencedBy(sentMessage)) {
-                                    conversation.addMessage(sentMessage);
-                                }
-                            }
-                        }
+                        Conversations.foldAndMergeWithList(conversations, sentMessages);
+                        sentMessages = null;
                     }
                 }
                 // Switch back folder
@@ -469,7 +462,13 @@ public final class IMAPConversationWorker {
                     try {
                         mailAccess = MailAccess.getInstance(ses, imapMessageStorage.getAccountId());
                         mailAccess.connect();
-                        IMAPStore imapStore = IMAPMessageStorage.getImapMessageStorageFrom(mailAccess).getImapStore();
+
+                        IMAPMessageStorage messageStorage = IMAPMessageStorage.getImapMessageStorageFrom(mailAccess);
+                        if (null == messageStorage) {
+                            // Eh...?
+                            throw new OXException(new IllegalStateException("Couldn't extract IMAP message storage out of " + mailAccess.getClass().getName()));
+                        }
+                        IMAPStore imapStore = messageStorage.getImapStore();
 
                         if (null != first) {
                             fillMessagesStatic(first, fullName, sentFullName, mergeWithSent, usedFields, headerNames, isRev1, imapStore, imapMessageStorage.getImapServerInfo(), mailAccount);
@@ -684,12 +683,27 @@ public final class IMAPConversationWorker {
             {
                 // Switch folder
                 imapMessageStorage.openReadOnly(sentFullName);
-                final List<MailMessage> sentMessages = Conversations.messagesFor(imapMessageStorage.getImapFolder(), lookAhead, order, fp, imapMessageStorage.getImapServerInfo(), byEnvelope);
-                for (final Conversation conversation : conversations) {
-                    for (final MailMessage sentMessage : sentMessages) {
-                        if (conversation.referencesOrIsReferencedBy(sentMessage)) {
-                            conversation.addMessage(sentMessage);
+                List<MailMessage> sentMessages = Conversations.messagesFor(imapMessageStorage.getImapFolder(), lookAhead, order, fp, imapMessageStorage.getImapServerInfo(), byEnvelope);
+                if (false == sentMessages.isEmpty()) {
+                    // Filter messages already contained in conversations
+                    {
+                        Set<String> allMessageIds = new HashSet<>(conversations.size());
+                        for (final Conversation conversation : conversations) {
+                            conversation.addMessageIdsTo(allMessageIds);
                         }
+                        for (Iterator<MailMessage> iter = sentMessages.iterator(); iter.hasNext();) {
+                            MailMessage sentMessage = iter.next();
+                            if (allMessageIds.contains(sentMessage.getMessageId())) {
+                                // Already contained
+                                iter.remove();
+                            }
+                        }
+                    }
+
+                    if (false == sentMessages.isEmpty()) {
+                        // Add to conversation if references or referenced-by
+                        Conversations.foldAndMergeWithList(conversations, sentMessages);
+                        sentMessages = null;
                     }
                 }
                 // Switch back folder
@@ -807,7 +821,7 @@ public final class IMAPConversationWorker {
     }
 
     private Comparator<List<MailMessage>> getListComparator(final MailSortField sortField, final OrderDirection order, final String fullName, final Locale locale) {
-        final MailMessageComparator comparator = new MailMessageComparator(sortField, OrderDirection.DESC.equals(order), locale);
+        final MailMessageComparator comparator = new MailMessageComparator(sortField, OrderDirection.DESC.equals(order), locale, imapFolderStorage.getImapConfig().getIMAPProperties().isUserFlagsEnabled());
         Comparator<List<MailMessage>> listComparator = new Comparator<List<MailMessage>>() {
 
             @Override
@@ -1034,7 +1048,7 @@ public final class IMAPConversationWorker {
     }
 
     private Comparator<MailThread> getThreadComparator(final MailSortField sortField, final OrderDirection order, Locale locale) {
-        final MailMessageComparator comparator = new MailMessageComparator(sortField, OrderDirection.DESC.equals(order), locale);
+        final MailMessageComparator comparator = new MailMessageComparator(sortField, OrderDirection.DESC.equals(order), locale, imapFolderStorage.getImapConfig().getIMAPProperties().isUserFlagsEnabled());
         Comparator<MailThread> threadComparator = new Comparator<MailThread>() {
 
             @Override
@@ -1079,7 +1093,7 @@ public final class IMAPConversationWorker {
                 /*
                  * Preselect message list according to given search pattern
                  */
-                filter = IMAPSearch.searchMessages(imapMessageStorage.getImapFolder(), searchTerm, imapMessageStorage.getImapConfig());
+                filter = IMAPSearch.searchMessages(imapMessageStorage.getImapFolder(), searchTerm, imapMessageStorage.getImapConfig(), imapMessageStorage.session);
                 if ((filter == null) || (filter.length == 0)) {
                     return IMAPMessageStorage.EMPTY_RETVAL;
                 }
@@ -1163,7 +1177,7 @@ public final class IMAPConversationWorker {
                 /*
                  * Sort according to order direction
                  */
-                Collections.sort(structuredList, new MailMessageComparator(effectiveSortField, descending, imapMessageStorage.getLocale()));
+                Collections.sort(structuredList, new MailMessageComparator(effectiveSortField, descending, imapMessageStorage.getLocale(), imapMessageStorage.getIMAPProperties().isUserFlagsEnabled()));
                 /*
                  * Output as flat list
                  */
@@ -1244,7 +1258,7 @@ public final class IMAPConversationWorker {
             /*
              * Sort according to order direction
              */
-            Collections.sort(structuredList, new MailMessageComparator(effectiveSortField, descending, imapMessageStorage.getLocale()));
+            Collections.sort(structuredList, new MailMessageComparator(effectiveSortField, descending, imapMessageStorage.getLocale(), imapMessageStorage.getIMAPProperties().isUserFlagsEnabled()));
             /*
              * Output as flat list
              */
