@@ -82,6 +82,7 @@ import com.openexchange.java.Strings;
 import com.openexchange.mail.MailSessionCache;
 import com.openexchange.mail.MailSessionParameterNames;
 import com.openexchange.mail.api.MailAccess;
+import com.openexchange.mail.config.MailProperties;
 import com.openexchange.mail.dataobjects.MailFolder;
 import com.openexchange.mail.dataobjects.MailFolder.DefaultFolderType;
 import com.openexchange.mail.mime.MimeMailException;
@@ -243,8 +244,6 @@ public class IMAPDefaultFolderChecker {
                     return DefaultFolderType.SPAM;
                 case StorageUtility.INDEX_TRASH:
                     return DefaultFolderType.TRASH;
-                default:
-                    break;
                 }
             }
         }
@@ -279,12 +278,12 @@ public class IMAPDefaultFolderChecker {
         if (null != arr) {
             if (StorageUtility.INDEX_CONFIRMED_HAM == index) {
                 SpamHandler spamHandler = SpamHandlerRegistry.getSpamHandlerBySession(session, accountId);
-                if (!spamHandler.isCreateConfirmedHam()) {
+                if (!spamHandler.isCreateConfirmedHam(session)) {
                     return retval;
                 }
             } else if (StorageUtility.INDEX_CONFIRMED_SPAM == index) {
                 SpamHandler spamHandler = SpamHandlerRegistry.getSpamHandlerBySession(session, accountId);
-                if (!spamHandler.isCreateConfirmedSpam()) {
+                if (!spamHandler.isCreateConfirmedSpam(session)) {
                     return retval;
                 }
             }
@@ -355,7 +354,7 @@ public class IMAPDefaultFolderChecker {
                                 /*
                                  * Strange... No INBOX available. Try to create it.
                                  */
-                                char sep = IMAPCommandsCollection.getSeparator(tmp);
+                                char sep = IMAPCommandsCollection.getSeparator(tmp, MailProperties.getInstance().getDefaultSeparator());
                                 try {
                                     IMAPCommandsCollection.createFolder(tmp, sep, FOLDER_TYPE);
                                 } catch (MessagingException e) {
@@ -635,14 +634,14 @@ public class IMAPDefaultFolderChecker {
 
                 // Check folder & return its full name
                 if (StorageUtility.INDEX_CONFIRMED_HAM == index) {
-                    if (spamHandler.isCreateConfirmedHam()) {
-                        checkedFullName = checkFullNameFor(index, namespace, specialUseInfo, fullName, name, sep, type, spamHandler.isUnsubscribeSpamFolders() ? 0 : -1, modified, accountChanged);
+                    if (spamHandler.isCreateConfirmedHam(session)) {
+                        checkedFullName = checkFullNameFor(index, namespace, specialUseInfo, fullName, name, sep, type, spamHandler.isUnsubscribeSpamFolders(session) ? 0 : -1, modified, accountChanged);
                     } else {
                         LOG.debug("Skipping check for {} due to SpamHandler.isCreateConfirmedHam()=false", name);
                     }
                 } else if (StorageUtility.INDEX_CONFIRMED_SPAM == index) {
-                    if (spamHandler.isCreateConfirmedSpam()) {
-                        checkedFullName = checkFullNameFor(index, namespace, specialUseInfo, fullName, name, sep, type, spamHandler.isUnsubscribeSpamFolders() ? 0 : -1, modified, accountChanged);
+                    if (spamHandler.isCreateConfirmedSpam(session)) {
+                        checkedFullName = checkFullNameFor(index, namespace, specialUseInfo, fullName, name, sep, type, spamHandler.isUnsubscribeSpamFolders(session) ? 0 : -1, modified, accountChanged);
                     } else {
                         LOG.debug("Skipping check for {} due to SpamHandler.isCreateConfirmedSpam()=false", name);
                     }
