@@ -72,14 +72,8 @@ import java.util.concurrent.atomic.AtomicReference;
 import javax.mail.Folder;
 import javax.mail.MessagingException;
 import org.cliffc.high_scale_lib.NonBlockingHashMap;
-import com.openexchange.config.ConfigurationService;
-import com.openexchange.config.Interests;
-import com.openexchange.config.Reloadable;
-import com.openexchange.config.Reloadables;
 import com.openexchange.exception.OXException;
 import com.openexchange.imap.IMAPCommandsCollection;
-import com.openexchange.imap.config.IMAPReloadable;
-import com.openexchange.imap.services.Services;
 import com.openexchange.java.Strings;
 import com.openexchange.log.LogProperties;
 import com.openexchange.mail.mime.MimeMailException;
@@ -439,52 +433,6 @@ final class ListLsubCollection implements Serializable {
         }
     }
 
-    private static volatile Integer initAclThreshold;
-    private static int initAclThreshold() {
-        Integer tmp = initAclThreshold;
-        if (null == tmp) {
-            synchronized (ListLsubCollection.class) {
-                tmp = initAclThreshold;
-                if (null == tmp) {
-                    final ConfigurationService service = Services.getService(ConfigurationService.class);
-                    tmp = Integer.valueOf(null == service ? 500 : service.getIntProperty("com.openexchange.imap.initAclThreshold", 500));
-                }
-            }
-        }
-        return tmp.intValue();
-    }
-
-    private static volatile Integer initStatusThreshold;
-    private static int initStatusThreshold() {
-        Integer tmp = initStatusThreshold;
-        if (null == tmp) {
-            synchronized (ListLsubCollection.class) {
-                tmp = initStatusThreshold;
-                if (null == tmp) {
-                    final ConfigurationService service = Services.getService(ConfigurationService.class);
-                    tmp = Integer.valueOf(null == service ? 500 : service.getIntProperty("com.openexchange.imap.initStatusThreshold", 500));
-                }
-            }
-        }
-        return tmp.intValue();
-    }
-
-    static {
-        IMAPReloadable.getInstance().addReloadable(new Reloadable() {
-
-            @Override
-            public void reloadConfiguration(final ConfigurationService configService) {
-                initAclThreshold = null;
-                initStatusThreshold = null;
-            }
-
-            @Override
-            public Interests getInterests() {
-                return Reloadables.interestsForProperties("com.openexchange.imap.initStatusThreshold", "com.openexchange.imap.initAclThreshold");
-            }
-        });
-    }
-
     private void init(boolean clearMaps, IMAPFolder imapFolder, final boolean ignoreSubscriptions, IMAPStore imapStore) throws MessagingException {
         if (clearMaps) {
             listMap.clear();
@@ -898,13 +846,22 @@ final class ListLsubCollection implements Serializable {
                     mbox = Boolean.FALSE;
                 }
                 final String fullName = listLsubEntry.getFullName();
-
+                final String originalFullName = listLsubEntry.optOriginalFullName();
                 // (Re-)Set children
                 {
-                    final ListLsubEntryImpl oldEntry = map.get(fullName);
+                    ListLsubEntryImpl oldEntry = map.get(fullName);
+                    if (oldEntry == null && originalFullName != null) {
+                        // Re-lookup with original full name
+                        oldEntry = map.get(originalFullName);
+                    }
+
                     if (oldEntry == null) {
                         // Wasn't in map before
                         map.put(fullName, listLsubEntry);
+                        if (originalFullName != null) {
+                            map.put(originalFullName, listLsubEntry);
+                        }
+
                     } else {
                         // Already contained in map
                         oldEntry.clearChildren();
@@ -2011,6 +1968,11 @@ final class ListLsubCollection implements Serializable {
             return fullName;
         }
 
+        @Override
+        public String optOriginalFullName() {
+            return null;
+        }
+
         @Override
         public Set<String> getAttributes() {
             return Collections.emptySet();
@@ -2108,6 +2070,8 @@ final class ListLsubCollection implements Serializable {
 
         private final String fullName;
 
+        private final String originalFullName;
+
         private Set<String> attributes;
 
         private char separator;
@@ -2136,7 +2100,10 @@ final class ListLsubCollection implements Serializable {
 
         protected ListLsubEntryImpl(final String fullName, final Set<String> attributes, final char separator, final ChangeState changeState, final boolean hasInferiors, final boolean canOpen, final Boolean hasChildren, final ConcurrentMap<String, ListLsubEntryImpl> lsubMap) {
             super();
-            this.fullName = checkFullName(fullName, separator);
+            String checkedFullName = checkFullName(fullName, separator);
+            this.fullName = checkedFullName;
+            this.originalFullName = checkedFullName.length() == 0 /* root full-name is the empty string */ || fullName.equals(checkedFullName) ? null : fullName;
+
             this.attributes = attributes;
             this.separator = separator;
             this.changeState = changeState;
@@ -2158,6 +2125,7 @@ final class ListLsubCollection implements Serializable {
         protected ListLsubEntryImpl(final ListLsubEntryImpl newEntry, final boolean subscribed) {
             super();
             fullName = newEntry.fullName;
+            originalFullName = newEntry.originalFullName;
             attributes = newEntry.attributes;
             canOpen = newEntry.canOpen;
             changeState = newEntry.changeState;
@@ -2337,6 +2305,11 @@ final class ListLsubCollection implements Serializable {
             return fullName;
         }
 
+        @Override
+        public String optOriginalFullName() {
+            return originalFullName;
+        }
+
         @Override
         public Set<String> getAttributes() {
             return attributes;
@@ -2577,7 +2550,7 @@ final class ListLsubCollection implements Serializable {
         if (null == fullName) {
             return fullName;
         }
-        if (String.valueOf(separator).equals(fullName)) {
+        if (fullName.length() == 1 && fullName.charAt(0) == separator) {
             return ROOT_FULL_NAME;
         }
         final String upperCase = Strings.toUpperCase(fullName);
