@@ -102,6 +102,13 @@ public interface ListLsubEntry {
      */
     String getFullName();
 
+    /**
+     * Gets the original full name (present in case the full name advertised by IMAP server is different from chosen one)
+     *
+     * @return The original full name or <code>null</code>
+     */
+    String optOriginalFullName();
+
     /**
      * Gets the attributes
      *
