@@ -78,7 +78,6 @@ import com.openexchange.imap.entity2acl.Entity2ACLExceptionCode;
 import com.openexchange.imap.entity2acl.IMAPServer;
 import com.openexchange.mail.MailSessionCache;
 import com.openexchange.mail.MailSessionParameterNames;
-import com.openexchange.mail.config.MailProperties;
 import com.openexchange.mail.dataobjects.MailFolder;
 import com.openexchange.mail.dataobjects.MailFolder.DefaultFolderType;
 import com.openexchange.mail.mime.MimeMailException;
@@ -109,8 +108,8 @@ public final class IMAPFolderConverter {
         private final int sessionUser;
         private final String fullname;
         private final char separator;
-        private final String otherUserNamespace;
-        private final String publicNamespace;
+        private final String[] otherUserNamespaces;
+        private final String[] publicNamespaces;
 
         /**
          * Initializes a new {@link Entity2ACLArgsImpl}.
@@ -120,7 +119,8 @@ public final class IMAPFolderConverter {
          * @param sessionUser The session user ID
          * @param fullname The IMAP folder's full name
          * @param separator The separator character
-         * @param otherUserNamespaces The namespace for other users
+         * @param otherUserNamespaces The namespaces for other users
+         * @param publicNamespaces The namespaces for public folders
          */
         Entity2ACLArgsImpl(int accountId, String serverUrl, int sessionUser, String fullname, char separator, String[] otherUserNamespaces, String[] publicNamespaces) {
             super();
@@ -129,13 +129,13 @@ public final class IMAPFolderConverter {
             this.sessionUser = sessionUser;
             this.fullname = fullname;
             this.separator = separator;
-            this.otherUserNamespace = null == otherUserNamespaces || otherUserNamespaces.length == 0 ? null : otherUserNamespaces[0];
-            this.publicNamespace = null == publicNamespaces || publicNamespaces.length == 0 ? null : publicNamespaces[0];
+            this.otherUserNamespaces = null == otherUserNamespaces || otherUserNamespaces.length == 0 ? null : otherUserNamespaces;
+            this.publicNamespaces = null == publicNamespaces || publicNamespaces.length == 0 ? null : publicNamespaces;
         }
 
         @Override
         public Object[] getArguments(final IMAPServer imapServer) throws OXException {
-            return imapServer.getArguments(accountId, serverUrl, sessionUser, fullname, separator, otherUserNamespace, publicNamespace);
+            return imapServer.getArguments(accountId, serverUrl, sessionUser, fullname, separator, otherUserNamespaces, publicNamespaces);
         }
     }
 
@@ -379,7 +379,7 @@ public final class IMAPFolderConverter {
                      * Check reliably for subscribed subfolders through LSUB command since folder attributes need not to to be present as
                      * per RFC 3501
                      */
-                    mailFolder.setSubscribedSubfolders(ListLsubCache.hasAnySubscribedSubfolder(imapFullName, accountId, imapFolder, session, imapConfig.getIMAPProperties().isIgnoreSubscription()));
+                    mailFolder.setSubscribedSubfolders(mailFolder.hasSubfolders() && ListLsubCache.hasAnySubscribedSubfolder(imapFullName, accountId, imapFolder, session, imapConfig.getIMAPProperties().isIgnoreSubscription()));
                 }
                 /*
                  * Set full name, name, and parent full name
@@ -450,11 +450,11 @@ public final class IMAPFolderConverter {
                     if (!exists || mailFolder.isNonExistent()) {
                         ownPermission.parseRights((ownRights = new Rights()), imapConfig);
                     } else if (!selectable) {
-                        ownRights = ownRightsFromProblematic(session, imapAccess, imapFullName, imapConfig, mailFolder, accountId, ownPermission);
+                        ownRights = ownRightsFromProblematic(session, imapAccess, imapFullName, imapConfig, mailFolder, accountId, ownPermission, listEntry);
                     } else {
                         ownRights = getOwnRights(imapFolder, session, imapConfig);
                         if (null == ownRights) {
-                            ownRights = ownRightsFromProblematic(session, imapAccess, imapFullName, imapConfig, mailFolder, accountId, ownPermission);
+                            ownRights = ownRightsFromProblematic(session, imapAccess, imapFullName, imapConfig, mailFolder, accountId, ownPermission, listEntry);
                         } else {
                             ownPermission.parseRights(ownRights, imapConfig);
                         }
@@ -499,7 +499,7 @@ public final class IMAPFolderConverter {
                     mailFolder.setUnreadMessageCount(-1);
                     mailFolder.setDeletedMessageCount(-1);
                 }
-                mailFolder.setSubscribed(MailProperties.getInstance().isSupportSubscription() ? ("INBOX".equals(mailFolder.getFullname()) ? true : listEntry.isSubscribed()) : true);
+                mailFolder.setSubscribed(imapConfig.getIMAPProperties().isSupportSubscription() ? ("INBOX".equals(mailFolder.getFullname()) ? true : listEntry.isSubscribed()) : true);
                 /*
                  * Parse ACLs to user/group permissions for primary account only.
                  */
@@ -519,7 +519,7 @@ public final class IMAPFolderConverter {
                 } else {
                     addOwnACL(mailFolder);
                 }
-                if (MailProperties.getInstance().isUserFlagsEnabled() && exists && selectable && imapConfig.getACLExtension().canRead(
+                if (imapConfig.getIMAPProperties().isUserFlagsEnabled() && exists && selectable && imapConfig.getACLExtension().canRead(
                     ownRights) && UserFlagsCache.supportsUserFlags(imapFolder, true, session, accountId)) {
                     mailFolder.setSupportsUserFlags(true);
                 } else {
@@ -555,7 +555,7 @@ public final class IMAPFolderConverter {
         return false;
     }
 
-    private static Rights ownRightsFromProblematic(Session session, IMAPAccess imapAccess, String imapFullName, IMAPConfig imapConfig, IMAPMailFolder mailFolder, int accountId, ACLPermission ownPermission) throws MessagingException, OXException, IMAPException {
+    private static Rights ownRightsFromProblematic(Session session, IMAPAccess imapAccess, String imapFullName, IMAPConfig imapConfig, IMAPMailFolder mailFolder, int accountId, ACLPermission ownPermission, ListLsubEntry listEntry) throws MessagingException, OXException, IMAPException {
         Rights ownRights;
         /*
          * Distinguish between holds folders and none
@@ -569,7 +569,13 @@ public final class IMAPFolderConverter {
                 ownPermission.parseRights(ownRights, imapConfig);
             } else {
                 ownPermission.setAllPermission(OCLPermission.CREATE_SUB_FOLDERS, OCLPermission.NO_PERMISSIONS, OCLPermission.NO_PERMISSIONS, OCLPermission.NO_PERMISSIONS);
-                ownPermission.setFolderAdmin(false);
+                // Allow folder administration if there are no subfolders...
+                if (listEntry.hasChildren()) {
+                    ownPermission.setFolderAdmin(false);
+                } else {
+                    // In case no children exist, the non-selectable should be manageable by user. Otherwise no chance to get rid off it...
+                    ownPermission.setFolderAdmin(true);
+                }
                 ownRights = ACLPermission.permission2Rights(ownPermission, imapConfig);
             }
         } else {
@@ -640,7 +646,7 @@ public final class IMAPFolderConverter {
              * Check if subfolder creation is allowed
              */
             final ACLPermission ownPermission = new ACLPermission();
-            final int fp = RootSubfoldersEnabledCache.isRootSubfoldersEnabled(imapConfig, rootFolder) ? OCLPermission.CREATE_SUB_FOLDERS : OCLPermission.READ_FOLDER;
+            final int fp = RootSubfoldersEnabledCache.isRootSubfoldersEnabled(imapConfig, rootFolder, session) ? OCLPermission.CREATE_SUB_FOLDERS : OCLPermission.READ_FOLDER;
             ownPermission.setEntity(session.getUserId());
             ownPermission.setAllPermission(fp, OCLPermission.NO_PERMISSIONS, OCLPermission.NO_PERMISSIONS, OCLPermission.NO_PERMISSIONS);
             ownPermission.setFolderAdmin(false);
