@@ -50,10 +50,8 @@
 package com.openexchange.imap.storecache;
 
 import static com.openexchange.imap.IMAPAccess.doIMAPConnect;
-import javax.mail.AuthenticationFailedException;
 import javax.mail.MessagingException;
 import com.openexchange.imap.IMAPClientParameters;
-import com.openexchange.log.LogProperties;
 import com.openexchange.session.Session;
 import com.sun.mail.imap.IMAPStore;
 
@@ -90,7 +88,7 @@ public abstract class AbstractIMAPStoreContainer implements IMAPStoreContainer {
      * @param login The login
      * @param pw The password
      * @param imapSession The IMAP session
-     * @param session The Groupware session
+     * @param session The Open-Xchange session
      * @return The newly created & connected {@link IMAPStore} instance
      * @throws MessagingException If operation fails
      */
@@ -106,22 +104,7 @@ public abstract class AbstractIMAPStoreContainer implements IMAPStoreContainer {
         /*
          * ... and connect it
          */
-        try {
-            doIMAPConnect(imapSession, imapStore, server, port, login, pw, accountId, session, false);
-        } catch (final AuthenticationFailedException e) {
-            /*
-             * Retry connect with AUTH=PLAIN disabled
-             */
-            imapSession.getProperties().put("mail.imap.auth.login.disable", "true");
-            imapStore = (IMAPStore) imapSession.getStore(name);
-            doIMAPConnect(imapSession, imapStore, server, port, login, pw, accountId, session, false);
-        }
-
-        String sessionInformation = imapStore.getClientParameter(IMAPClientParameters.SESSION_ID.getParamName());
-        if (null != sessionInformation) {
-            LogProperties.put(LogProperties.Name.MAIL_SESSION, sessionInformation);
-        }
-
+        doIMAPConnect(imapSession, imapStore, server, port, login, pw, accountId, session, false);
         return imapStore;
     }
 
